require 'json'
module Ohai
  module Sas2ircu
    # Represents a Sas Card
    class Card
      attr_accessor :id, :backplanes, :drives_by_serial, :drives_by_wnn

      CARD_REGEX = /\s{3}Enclosure#\s+:\s(\d+)\s{3}Logical ID\s+:\s([0-9a-f]+:[0-9a-f]+)\s{3}Numslots\s+:\s(\d+)\s{3}StartSlot\s+:\s(\d+)/.freeze
      BACKPLANE_REGEX = /Device is a Hard disk\s{3}Enclosure #\s+:\s(\d+)\s{3}Slot #\s+:\s(\d+)\s{3}SAS Address\s+:\s([a-f0-9\-]+)\s{3}State\s+:\s(.+)\s{3}Size \(in MB\)\/\(in sectors\)\s+:\s([0-9]+)\/([0-9]+)\s{3}Manufacturer\s+:\s(.+)\s{3}Model Number\s+:\s(.+)\s{3}Firmware Revision\s+:\s(.+)\s{3}Serial No\s+:\s(.+)\s{3}GUID\s+:\s([0-9a-f]+)\s+Protocol\s+:\s(.+)\s{3}Drive Type\s+:\s(.+)/.freeze
      def initialize(id)
        self.id = id
        self.backplanes = {}
        self.drives_by_serial = {}
        self.drives_by_wnn = {}
        populate_backplanes
      end

      def to_h
        bs = {}
        backplanes.each do |_k, b|
          bs[b.enclosure] = b.to_h
        end
        { backplanes: bs }
      end

      def to_mash
        bs = Mash.new
        backplanes.each do |_k, b|
          bs[b.enclosure] = b.to_mash
        end
        c = Mash.new
        c[:backplanes] = bs
        c
      end

      protected

      def sas2ircu_lines
        @lines ||= Ohai::Mixin::Command.shell_out("grab_sas2ircu_display.sh #{id}").stdout
      end

      def populate_backplanes
        sas2ircu_lines.scan(CARD_REGEX).each do |bp|
          id = bp[0].to_i
          backplanes[id] = Backplane.new self, bp[0], bp[1], bp[2], bp[3]
        end
        populate_slots
      end

      def populate_slots
        sas2ircu_lines.scan(BACKPLANE_REGEX).each do |d|
          b = @backplanes[d[0].to_i]
          disk = Disk.new b, d
          b.slots[d[1].to_i] = disk
          drives_by_serial[d[9]] = disk
          drives_by_wnn[d[10]] = disk
        end
      end
    end

    # Represents a backplane (eg front or rear of jbod)
    class Backplane
      attr_accessor :card, :enclosure,
                    :logical_id, :num_slots, :start_slot, :slots
      def initialize(card = nil, enclosure = nil, logical_id = nil,
                     num_slots = nil, start_slot = nil)
        self.card = card
        self.enclosure = enclosure.to_i
        self.logical_id = logical_id.to_i
        self.num_slots = num_slots.to_i
        self.start_slot = start_slot.to_i
        self.slots = {}
      end

      def expand_slots
        (start_slot..start_slot + num_slots - 1).each do |slot_number|
          slots[slot_number] = Disk.new(self) unless slots[slot_number]
        end
        self
      end

      def to_h
        ss = {}
        slots.each do |_k, s|
          ss[s.slot] = s.to_h
        end
        { enclosure: enclosure, logical_id: logical_id, num_slots: num_slots,
          start_slot: start_slot, slots: ss }
      end

      def to_mash
        ss = Mash.new
        slots.each do |_k, s|
          ss[s.slot] = s.to_mash
        end
        resp = Mash.new
        to_h.each { |k, v| resp[k] = v }
        resp[:slots] = ss
        resp
      end
    end

    # Represents a Disk
    class Disk
      attr_accessor :backplane
      def initialize(backplane, disk_array = nil)
        self.backplane = backplane
        @array = disk_array
      end

      def empty?
        @array.nil?
      end

      def slot
        @array[1]
      end

      def sas_address
        @array[2]
      end

      def state
        @array[3]
      end

      def size_mb
        @array[4]
      end

      def size_sectors
        @array[5]
      end

      def manufacturer
        @array[6]
      end

      def model_number
        @array[7]
      end

      def firmware_revision
        @array[8]
      end

      def serial_no
        @array[9]
      end

      def guid
        @array[10]
      end

      def protocol
        @array[11]
      end

      def drive_type
        @array[12]
      end

      def to_h
        {
          slot: slot, state: state, size_mb: size_mb,
          size_sectors: size_sectors, drive_type: drive_type,
          sas_address: sas_address, manufacturer: manufacturer,
          model_number: model_number, firmware_revision: firmware_revision,
          serial_no: serial_no, guid: guid, protocol: protocol
        }
      end

      def to_mash
        m = Mash.new
        to_h.each { |k, v| m[k] = v }
        m
      end
    end

    # Parse the contents of the Sas Cards
    class Parser
      attr_accessor :cards

      def initialize
        self.cards = {}
        populate
      end

      def number_of_cards
        @no_cards ||= Ohai::Mixin::Command.shell_out('sas2ircu list | grep Index| wc -l').stdout
        @no_cards = @no_cards.delete("\n").to_i
      end

      def populate
        (0..(number_of_cards - 1)).each do |card_id|
          cards[card_id] = Card.new(card_id)
        end
      end

      def to_h
        cr = {}
        cards.each do |_k, c|
          cr[c.id] = c.to_h
        end
        { cards: cr }
      end

      def to_mash
        cr = Mash.new
        cards.each do |_k, c|
          cr[c.id] = c.to_mash
        end
        m = Mash.new
        m[:cards] = cr
        m
      end
    end
  end
end

Ohai.plugin(:Sas2ircu) do
  provides 'sas2ircu'
  collect_data :default do
    Ohai::Log.debug('Loading data from sas2ircu')
    sas2ircu Ohai::Sas2ircu::Parser.new.to_mash
  end
end
